use strict;
use warnings;

use Module::Build;

my $build = Module::Build->new(
	module_name => 'App::Ground',
	dist_author => 'Eugen Konkov',
	requires => {
		'DateTime'                      =>  0,
		'DateTime::HiRes'               =>  0,
		'DateTime::Format::Pg'          =>  0,
		'DateTime::Format::Epoch::Unix' =>  0,
		'Email::Stuffer'                =>  0,
		'MIME::Base64'                  =>  0,
		'Authen::SASL'                  =>  0,
		'Math::BigFloat'                =>  0,
	},
	recommends         => {
	},
	test_requires      => {
	},
	configure_requires => {
		'Module::Build' =>  0,
	},
	license            => 'MIT',
	meta_merge         => {
		resources => {
			repository =>  'https://github.com/KES777/App-Ground',
			bugtracker =>  'https://github.com/KES777/App-Ground/issues',
		},
	},
);

$build->create_build_script;
