package # S - because singleton
	S;  # Hide from PAUSE

use strict;
use warnings;



1;

=encoding utf8

=head1 NAME

C<S> - singleton namespace

=head1 SYNOPSIS

  use S;

=head1 DESCRIPTION

C<S> is an intentionally empty module. It provides a unique namespace that can
be used as a singleton holder for globals or state shared across the
application.

=head1 SEE ALSO

L<A>, L<C>, L<I>, L<L>, L<M>, L<S>, L<T>, L<U>.

=cut
