package # A - application
	A;  # Hide from PAUSE

use strict;
use warnings;



1;

=encoding utf8

=head1 NAME

C<A> - Application

=head1 SYNOPSIS

	# inside your application lib/A.pm
	package A;

	use lib 'ext/MojoDebug';
	use lib 'ext/LocalConfig';
	use lib 'ext/Util';

	use C;
	use L;
	use D;


	# then in your scripts/helpers or application
	use A; # Now you have the same environment

=head1 DESCRIPTION

This module exists to setup something global for your application. Prepare
namespaces, load libraries etc. Also you could define application scoped logic
which could be reused from anywhere, eg. bootstraping.
This is similar to `use Mojo::Base -base;` idea. See L<Mojo::Base>.

=head1 SEE ALSO

L<C>, L<I>, L<L>, L<M>, L<S>, L<T>, L<U>.

=cut
